/*  
	Innovations Project - David Walker NCCA 2000 
	File converter between Max and Maya
	Max2Maya v1.1 
	 
	Initial parser framework - Eike Anderson NCCA 2000 
*/ 
 
#include <stdio.h> 
#include <string.h> 
#include <ctype.h> 
#include <stdlib.h> 
#include <time.h>
#include <math.h>
 
#define KNOWN_TOKENS  43      // the number of known tokens 
#define CHAR_SETSIZE  256 
#define LINELEN      1025     // maximum allowed line length (including string terminator) 
#define CR           0x0D     // Carriage Return 
#define LF           0x0A     // Line Feed

#define SQUARE(X) X*X
#define RAD2DEG(X) X*57.29578
#define POSOFFSET 10
 
//  all the symbols/tokens that can occurr, including none (NIL) and all special characters 
enum TOKEN 
{ 
  NIL, 
  TDSMAX_ASCIIEXPORT, COMMENT, GEOMOBJECT, 
  NODE_NAME, MESH, MESH_NUMVERTEX, MESH_NUMFACES, 
  MESH_VERTEX_LIST, MESH_VERTEX, MESH_FACE_LIST, 
  MESH_FACE, MESH_SMOOTHING, MESH_MTLID, 
  NODE_TM, MESH_NORMALS, LIGHTOBJECT, CAMERAOBJECT, 
  TM_POS, TM_SCALE, LIGHT_SETTINGS, LIGHT_COLOR, 
  LIGHT_INTENS, LIGHT_FALLOFF, LIGHT_HOTSPOT, 
  CAMERA_SETTINGS, CAMERA_NEAR, CAMERA_FAR, CAMERA_FOV, 
  CAMERA_TDIST, TM_ANIMATION, CONTROL_POS_BEZIER, 
  CONTROL_BEZIER_POS_KEY, SCENE_TICKSPERFRAME, SCENE, 
  SCENE_FIRSTFRAME, SCENE_LASTFRAME, SCENE_FRAMESPEED, 
  CONTROL_SCALE_BEZIER,CONTROL_BEZIER_SCALE_KEY,
  CONTROL_ROT_TCB,CONTROL_TCB_ROT_KEY, TM_ROTAXIS,
  TM_ROTANGLE, 
  STAR, LBRACE, RBRACE, COLON, SPEECH, FULL, 
  REAL, INTEGER, STRING, MEOF 
};

// -------------------- STRUCTURE DEFNS FOR LISTS OF OBJECTS/CAMERAS/LIGHTS, ETC. ---- 
 
struct OLIST 
{ 
  char  name[LINELEN];  
  OLIST *next; 
}; 
 
struct LIGHTLIST 
{ 
	char name[LINELEN]; 
	LIGHTLIST *next; 
}; 
 
struct CAMERALIST 
{ 
	char name[LINELEN]; 
	CAMERALIST *next; 
}; 
 
struct ANIMLIST 
{ 
 	char name[LINELEN]; 
	ANIMLIST *next; 
}; 

struct ROTXYZ
{
	float x,y,z;
};
 
 
 
typedef char ALFA[129]; 
 
// -------------------- ALL THE TOKEN WE WANT TO WORK WITH... ----
// program searches thorugh the .ASE file comparing each symbol with 
// this token list. The program uses various switch/case statements
// to take the correct action on the file.

ALFA word[KNOWN_TOKENS]= 
{ 
  "3DSMAX_ASCIIEXPORT", "COMMENT", "GEOMOBJECT", 
  "NODE_NAME", "MESH", "MESH_NUMVERTEX", "MESH_NUMFACES", 
  "MESH_VERTEX_LIST", "MESH_VERTEX", "MESH_FACE_LIST", 
  "MESH_FACE", "MESH_SMOOTHING", "MESH_MTLID", 
  "NODE_TM", "MESH_NORMALS", "LIGHTOBJECT", "CAMERAOBJECT", 
  "TM_POS", "TM_SCALE", "LIGHT_SETTINGS", "LIGHT_COLOR", 
  "LIGHT_INTENS", "LIGHT_FALLOFF", "LIGHT_HOTSPOT", 
  "CAMERA_SETTINGS", "CAMERA_NEAR", "CAMERA_FAR", "CAMERA_FOV", 
  "CAMERA_TDIST", "TM_ANIMATION", "CONTROL_POS_BEZIER", 
  "CONTROL_BEZIER_POS_KEY", "SCENE_TICKSPERFRAME", "SCENE", 
  "SCENE_FIRSTFRAME", "SCENE_LASTFRAME", "SCENE_FRAMESPEED", "CONTROL_SCALE_BEZIER", 
  "CONTROL_BEZIER_SCALE_KEY","CONTROL_ROT_TCB","CONTROL_TCB_ROT_KEY", "TM_ROTAXIS",
  "TM_ROTANGLE" 
}; 
 
//  has to be same size and sequence as ^word^ 
TOKEN symbol[KNOWN_TOKENS]= 
{ 
  TDSMAX_ASCIIEXPORT, COMMENT, GEOMOBJECT, 
  NODE_NAME, MESH, MESH_NUMVERTEX, MESH_NUMFACES, 
  MESH_VERTEX_LIST, MESH_VERTEX, MESH_FACE_LIST, 
  MESH_FACE, MESH_SMOOTHING, MESH_MTLID, 
  NODE_TM, MESH_NORMALS, LIGHTOBJECT, CAMERAOBJECT, TM_POS, 
  TM_SCALE, LIGHT_SETTINGS, LIGHT_COLOR, LIGHT_INTENS, 
  LIGHT_FALLOFF, LIGHT_HOTSPOT, CAMERA_SETTINGS, CAMERA_NEAR, 
  CAMERA_FAR, CAMERA_FOV, CAMERA_TDIST, TM_ANIMATION, 
  CONTROL_POS_BEZIER, CONTROL_BEZIER_POS_KEY, SCENE_TICKSPERFRAME, SCENE, 
  SCENE_FIRSTFRAME, SCENE_LASTFRAME, SCENE_FRAMESPEED, CONTROL_SCALE_BEZIER, 
  CONTROL_BEZIER_SCALE_KEY,CONTROL_ROT_TCB,CONTROL_TCB_ROT_KEY, TM_ROTAXIS,
  TM_ROTANGLE
}; 
 
 
TOKEN special[CHAR_SETSIZE]={NIL};  // special characters 

// -------------------- VARIABLE AND FUNCTION DEFNS ----
 
FILE *OutPutFilePtr; 
FILE *InputFilePtr; 
char filename[256]; 
char outputfile[256]; 
char argnought[40];     // program name 
char line[LINELEN];     // current line of the InputFile 
char ch=' ';            // last read character 
TOKEN sym;              // last read symbol 
char  sval[LINELEN]; 
long int ival=0; 
double fval=0.0; 
ALFA  id;               // the current identifier 
 
OLIST *objList=NULL; 
LIGHTLIST *lightList=NULL; 
CAMERALIST *cameraList=NULL; 
ANIMLIST *animList=NULL; 
 
int i=0; 
int cc=0;       //  current character (in the current line) 
int ll=0;       //  current line length 
char eof=0; 
int inObj=0; 
int inLight=0; 
int inCamera=0; 
int inAnim=0; 
int ticks=0; 
int SF=0; 
int EF=0; 
int FS=0; 
int transkey=0; 
int scalekey=0; 
int rotkey=0;

  //int MaxCount=0;
  //int MaxCount1=0;

int Anim=0, Geom=0, Light=0, Camera=0, Uni=0, Scene=0;
char version[]="1.1";

 
void message(int); 
void initialise(); 
void parse(); 
void object(); 
void mesh(); 
void mesh_vertex_list(); 
void mesh_face_list(); 
void top(); 
void bottom(); 
void WriteObjName(); 
void GetFaces(); 
void GetNumObj(); 
void GetCh(); 
void GetSym(); 
void animobj(); 
void sceneinfo();
void help();
ROTXYZ Quat2Deg(float, float, float, float);
 
void lightobject(); 
void cameraobject(); 
 
char name[256]; 
 
char buf[256]; 
char temp[256]; 
int Number; 
int FaceNumber=0; 
int VertexNumber=0; 
 
int TEST=0;
char deffile[]="test.ase"; 
char defout[]="OutTest.ma"; 
 
long lineno=0; 
long fileSize; 
 
float **VertPos; 
int   **Faces;
char helpme[]="-h";

void main (int argc, char *argv[]) 
{ 
  strcpy(argnought,argv[0]);
  initialise(); 
 
	if (TEST==1) 
	{ 
	 strcpy(filename, deffile); 
	 strcpy(outputfile, defout);
	 Uni=1;
	} 

// -------------------- CHECK AND SET PARAMETERS : ----
// Sets some default filenames for testing purposes
// Also tests argc and argv for number and value of params
// passed into the program

  if ((argc==3)&&(TEST==0))
 Uni=1;

 if ((argc==2)&&(TEST==0))
 {
 if ( (strcmp(helpme, argv[1]) == 0) )
	{
	help();
	}
}
 //copy by default - required params
if (TEST==0)
{
 if (argc < 3)
	{
	message(1);
	}
 else
	{
 strcpy(filename,argv[1]);
 strcpy(outputfile,argv[2]);
	}
}

if ((argc>3)&&(TEST==0))
{

for (i=3; i<argc; i++)
 {
 switch (argv[i][1])
  {
	case 'a':case 'A': Anim=1;Scene=1;Geom=1;
				      break;
   case 'g':case 'G': Geom=1;Scene=1;
				      break;
   case 'l':case 'L': Light=1;
				      break;
   case 'c':case 'C': Camera=1;
				      break;
   case 'u':case 'U': Uni=1;
						break;
	case 's':case 'S': Scene=1;
						break;
	case 'h':case 'H': help();
						break;


	default:    printf("Unrecognized parameter : -%c\n", argv[i][1]);
				message(1);
				exit(EXIT_FAILURE);
				break;
  }
 }
}

// If no params passed to the prog set all values to 1, i.e universal 

if (Uni==1)
{
Anim=1;
Geom=1;
Light=1;
Camera=1;
Scene=1;
}
 
  if ( (OutPutFilePtr = fopen(outputfile, "w")) == NULL) 
	message(6); 
  if ( (InputFilePtr = fopen(filename, "r")) == NULL) 
	message(7); 

printf("\n");
printf("Entering files\n"); 
 
// --------------- FIRST STEP : VERIFY VALIDITY OF THE INPUT FILE
// Are the first few symbols == TDSMAX_ASCIIEXPORT?
// If yes -> file is of the correct type - if no -> invalid input file

  int symcnt=0;   // emergency counter to throw out non-MAX files
 
  do{
     if(symcnt>10)
       message(5);
	 GetSym();
     symcnt++;
  }while(sym!=TDSMAX_ASCIIEXPORT);  
  GetSym(); 
	 if(sym!=INTEGER) 
	 message(5); 
  if(ival!=200) 
	 message(5); 
  GetSym(); 
  // done with that - on with the show... 
 
  //  write header of Maya file 
  top(); 
 
  // parse the file for stuff 
  parse(); 
 
  //  write footer of Maya file 
  bottom(); 
 
 
  fclose(OutPutFilePtr); 
  fclose(InputFilePtr);
  printf ("\nThankyou for using Max2Maya %s\n", version);
}

// -------------------- PARSING FUNCTION ---- 
 
void parse() 
{ 

int count=0,count2=0, lightcount=0, cameracount=0; 
  do 
  {  
	 count2++; 
	 //printf("loop #%d\n",count2); 
	 fflush(stdout); 
	 if(sym==MEOF||eof) 
		break; 
	 GetSym(); 
 
	 if ( (sym==SCENE)&&(Scene==1) )
	 { 
	 printf("Found Scene info\n"); 
	 sceneinfo(); 
	 } 
	 else 
	 if ( (sym==GEOMOBJECT)&&(Geom==1) )
    { 
		count++; 
		//printf("Entering object number %d\n",count); 
		object();
		printf("Entering Object : %s\n", name); 
		//printf("exited object number %d - eof=%d\n",count,eof);
		fflush(stdout); 
	 } 
	 else 
	 if ( (sym==LIGHTOBJECT)&&(Light==1) )
	 { 
	 lightcount++; 
	 //printf ("Entering light number : %d\n", lightcount);
	 lightobject(); 
	 fflush(stdout); 
	 } 
	 else 
	 if ( (sym==CAMERAOBJECT)&&(Camera==1) )
	 { 
	 cameracount++; 
	 //printf ("Entering camera number : %d\n", cameracount);
	 cameraobject(); 
	 fflush(stdout); 
	 } 
 
 
  }while(sym!=MEOF&&eof==0); 

fflush(stdout); 
}

// -------------------- OBJECT FUNCTION ---- 
// Function to take object info from ASE file. Switch/case tests against tokens declared
// at top of program and takes the correct action dependant on what the token is, for example,
// record and write the object name, record and write the object mesh, or enter an animation, etc.
 
void object() 
{ 
  inObj=1; 
  GetSym(); 
  if(sym!=LBRACE) 
	 message(4); 
  GetSym(); 
  while(sym!=RBRACE) 
  { 
	 switch(sym) 
	 { 
		case NODE_NAME: GetSym(); 
							 if(sym==STRING) 
								 WriteObjName(); 
							 GetSym();    
							 break; 
		case   NODE_TM: do                    // we'll ignore this 
								GetSym(); 
							 while(sym!=RBRACE); 
							 GetSym(); 
							 break; 
		case      MESH: mesh(); 
							 GetSym(); 
							 break; 
		case TM_ANIMATION:if(Anim==1)
								{
								printf ("Entering animation node\n");
								animobj();
								GetSym();
								break;
								}
								else
								{
								GetSym();
								break;
                        }
				 default: GetSym(); 
 
	 } 
	 if(eof||sym==MEOF) 
		break; 
  } 
  // we're finished with this object 
  inObj=0; 
 
fflush(stdout); 
} 

// -------------------- OBJECT MESH FUNCTION ---- 
 
void mesh() 
{ 
  GetSym(); 
  if(sym!=LBRACE) 
	 message(4); 
  GetSym(); 
  while(sym!=RBRACE) 
  { 
	 switch(sym) 
    { 
		case MESH_NUMVERTEX: GetSym(); 
                           if(sym==INTEGER) 
										VertexNumber=ival;
									else 
                              message(4); 
									VertPos=(float **)malloc(VertexNumber*sizeof(float*)); 
									for (i=0;i<VertexNumber;i++) 
										VertPos[i]=(float*)malloc(3*sizeof(float)); 
									GetSym(); 
									break; 
		case  MESH_NUMFACES: GetSym(); 
                           if(sym==INTEGER) 
										FaceNumber=ival; 
									else 
										message(4); 
									Faces=(int **)malloc(FaceNumber*sizeof(int*)); 
                           for (i=0;i<FaceNumber;i++) 
										Faces[i]=(int*)malloc(8*sizeof(int)); 
									GetSym(); 
                           break; 
		case MESH_VERTEX_LIST: mesh_vertex_list(); 
									break; 
		case MESH_FACE_LIST: mesh_face_list(); 
									break; 
      case   MESH_NORMALS: do                    // we'll ignore this 
									  GetSym(); 
									while(sym!=RBRACE); 
									GetSym(); 
									break; 
						default: GetSym(); 
	 } 
  } 
  // we're finished with this mesh 
  // now we can hopefully free the arrays... 
  for(i=0;i<FaceNumber;i++) 
	 free(Faces[i]); 
  free(Faces); 
  for(i=0;i<VertexNumber;i++) 
	 free(VertPos[i]); 
  free(VertPos); 
//printf("exit mesh\n"); 
//fflush(stdout); 
}

// -------------------- VERTEX LIST FUNCTION ----  
 
void mesh_vertex_list() 
{ 
 
 char vertblurb1[]="setAttr -s "; 
 char vertblurb2[]= "\".vt[0:"; 
 char vertblurb3[]="]\"\n"; 
 i=0; 
 GetSym(); 
 if(sym!=LBRACE) 
	message(4); 
 GetSym(); 
 do 
 { 
	 if(sym==MESH_VERTEX) 
	 { 
		GetSym(); 
		GetSym(); 
		VertPos[i][0]=fval; 
		GetSym(); 
		VertPos[i][2]=fval; 
		GetSym(); 
		VertPos[i][1]=fval; 
		 i++; 
	 } 
	 GetSym(); 
  } while(sym!=RBRACE); 
  fprintf (OutPutFilePtr, " %s%d %s%d%s", vertblurb1, VertexNumber, vertblurb2, (VertexNumber-1), vertblurb3); 
  for (i=0; i<VertexNumber; i++) 
  { 
	 fprintf (OutPutFilePtr, "                 %f %f %f\n", VertPos[i][0],VertPos[i][2],VertPos[i][1]); 
	 fflush(OutPutFilePtr); 
  } 
  fprintf (OutPutFilePtr, ";\n"); 
  fflush(OutPutFilePtr);        // make sure this gets written to file immediately 
 
 
 
fflush(stdout); 
 
  GetSym(); 
}

// -------------------- FACE LIST FUNCTION ----  
 
void mesh_face_list() 
{ 
 
char vertblurb1[]="setAttr -s "; 
char vertblurb2[]= "\".ed[0:"; 
char vertblurb3[]="]\"\n"; 
 i=0; 
 GetSym(); 
 if(sym!=LBRACE) 
	message(4); 
 GetSym(); 
do 
 { 
	 if(sym==MESH_FACE) 
	 { 
	 GetSym(); 
	 GetSym(); 
	 GetSym(); 
	 GetSym(); 
	 GetSym(); 
	 Faces[i][0]=ival; 
	 GetSym(); 
	 GetSym(); 
	 GetSym(); 
	 Faces[i][1]=ival; 
	 GetSym(); 
	 GetSym(); 
	 GetSym(); 
	 Faces[i][2]=ival; 
	 i++; 
	 } 
    GetSym(); 
  }while(sym!=RBRACE); 
 
fprintf (OutPutFilePtr, " %s%d %s%d%s ", vertblurb1, (FaceNumber*3), vertblurb2, ((FaceNumber*3)-1), vertblurb3); 
 
for (i=0; i<FaceNumber; i++) 
  { 
  fprintf (OutPutFilePtr, "                 %d %d %d\n", Faces[i][0],Faces[i][1],1); 
	 fprintf (OutPutFilePtr, "                 %d %d %d\n", Faces[i][1],Faces[i][2],1); 
		fprintf (OutPutFilePtr, "                 %d %d %d\n\n", Faces[i][2],Faces[i][0],1); 
  //printf ("Edges : %d : %d %d  -  %d %d  -  %d %d\n", i, Faces[i][0],Faces[i][1],Faces[i][1],Faces[i][2],Faces[i][2],Faces[i][0]);
  fflush(OutPutFilePtr); 
  } 
  fprintf (OutPutFilePtr, ";\n\n"); 
 
// FACE INFORMATION : 
 
char faceblurb1[]="setAttr -s "; 
char faceblurb2[]= "\".fc[0:"; 
char faceblurb3[]="]\" -type \"polyFaces\" \n\n"; 
char facestd[]="f 3"; 
 
fprintf (OutPutFilePtr, " %s%d %s%d%s ", faceblurb1, (FaceNumber), faceblurb2, (FaceNumber-1), faceblurb3); 
 
for (i=0; i<FaceNumber; i++) 
 { 
 fprintf (OutPutFilePtr, " %s %d %d %d\n", facestd, (i*3), ((i*3)+1), ((i*3)+2) ); 
 } 
  fprintf (OutPutFilePtr, ";\n\n"); 
 
  fflush(OutPutFilePtr); 
 
 GetSym(); 
 
}

// -------------------- LIGHT FUNCTION ----  
 
void lightobject() 
{ char ln[256];

float lightrotaxis[3];
float lightrotangle;


  inLight=1; 
  GetSym(); 
  if(sym!=LBRACE) 
	 message(4); 
  GetSym(); 
  while(sym!=RBRACE) 
  { 
	 switch(sym) 
	 { 
		case NODE_TM:  do{ 
							  switch (sym) 
								 { 
								 case NODE_NAME:char lightname[256]; 
													 GetSym(); 
													 if(sym==STRING) 
													 strcpy(lightname, sval);strcpy(ln, lightname);
													 
													 fprintf(OutPutFilePtr, "createNode transform -n \"%s\";\n", ln);
													 //printf("Name is %s\n", lightname); 
 
														LIGHTLIST *nlight; 
														LIGHTLIST *currentl; 
 
														  // now add this object to our list of objects... 
														nlight=(LIGHTLIST*)malloc(sizeof(LIGHTLIST)); 
														nlight->next=NULL; 
														strcpy(nlight->name,lightname); 
 
														currentl=lightList; 
														if(currentl!=NULL) 
														{ 
															while(currentl->next!=NULL) 
															currentl=currentl->next; 
															currentl->next=nlight; 
														} 
													  else 
														 lightList=nlight; 
 
 
													 GetSym(); 
													 break; 
								 case TM_POS:float lightpos[3]; 
												 GetSym(); 
												 lightpos[0]=fval; 
												 GetSym(); 
												 lightpos[1]=fval; 
												 GetSym(); 
												 lightpos[2]=fval; 
												 //GetSym(); 
												//printf ("XYZ trans Elements are : %f, %f, %f\n", lightpos[0], lightpos[2], lightpos[1]);
												fprintf(OutPutFilePtr, "	setAttr \".t\" -type \"double3\" %f %f %f ;\n",lightpos[0], lightpos[1], lightpos[2]); 
												GetSym(); 
												break;


								 case TM_ROTAXIS:GetSym(); 
												 lightrotaxis[0]=fval; 
												 GetSym(); 
												 lightrotaxis[1]=fval; 
												 GetSym(); 
												 lightrotaxis[2]=fval; 
												 GetSym(); 
												//printf ("Rot Axis Elements are : %f, %f, %f\n", lightrotaxis[0], lightrotaxis[1], lightrotaxis[2]);
												//GetSym(); 
												break;

								 case TM_ROTANGLE:GetSym();
												  ROTXYZ NEWROTATION; 
												  lightrotangle=fval; 
												  //printf ("Rot Axngle Element is : %f\n", lightrotangle);

												  NEWROTATION=Quat2Deg((lightrotaxis[0]),(lightrotaxis[1]),(lightrotaxis[2]),(lightrotangle));
												  //printf("Light XYZ Rotation angles are :\n X=%f, Y=%f, Z=%f",NEWROTATION.x,NEWROTATION.y,NEWROTATION.z);
												 fprintf(OutPutFilePtr, "	setAttr \".r\" -type \"double3\" %f %f %f ;\n",NEWROTATION.x,NEWROTATION.y,NEWROTATION.z); 

												  GetSym(); 
												  break;


 
								 case TM_SCALE:float lightscale[3]; 
												 GetSym(); 
												 lightscale[0]=fval; 
												 GetSym(); 
												 lightscale[1]=fval; 
												 GetSym(); 
												 lightscale[2]=fval; 
												 GetSym(); 
												//printf ("XYZ scale Elements are : %f, %f, %f\n", lightscale[0], lightscale[1], lightscale[2]);
												fprintf(OutPutFilePtr, "	setAttr \".s\" -type \"double3\" %f %f %f ;\n",lightscale[0], lightscale[1], lightscale[2]); 
												GetSym(); 
												break; 
								 default : GetSym(); 
								 } 
								}while (sym!=RBRACE); 
 
		  case LIGHT_SETTINGS:		LIGHTLIST *currentl; 
											currentl=lightList; 
 
											if(currentl->next != NULL) 
											 currentl=currentl->next; 
											fprintf (OutPutFilePtr, "createNode spotLight -n \"%sShape1\" -p \"%s\";\n ",ln,ln);
											//printf ("Light Name from LIGHTLIST = %s\n",currentl->name);
											fprintf(OutPutFilePtr, "	setAttr -k off \".v\";\n"); 
 
										 do{ 
										 switch (sym) 
											{ 
											case LIGHT_COLOR:float lightcol[3]; 
																	GetSym(); 
																	lightcol[0]=fval; 
																	GetSym(); 
																	lightcol[1]=fval; 
																	GetSym(); 
																	lightcol[2]=fval; 
																	GetSym(); 
																	//printf ("Colour Elements are : %f, %f, %f\n", lightcol[0], lightcol[1], lightcol[2]);
																	fprintf (OutPutFilePtr, "	setAttr \".cl\" -type \"float3\" %f %f %f ;\n",lightcol[0], lightcol[1], lightcol[2]); 
																	break; 
											case LIGHT_INTENS:float lightints; 
																	GetSym(); 
																	lightints=fval; 
																	//printf ("Light intensity is : %f\n", lightints);
																	fprintf (OutPutFilePtr, "	setAttr \".in\" %f ;\n",lightints); 
																	break; 
											case LIGHT_FALLOFF:float lightfall; 
																	GetSym(); 
																	lightfall=fval; 
																	//printf ("Light DropOff is : %f\n", lightfall);
																	fprintf (OutPutFilePtr, "	setAttr \".dro\" %f ;\n\n",lightfall); 
																	break; 
											case LIGHT_HOTSPOT:float coneangle; 
																	GetSym(); 
																	coneangle=fval; 
																	//printf ("Light ConeAngle is : %f\n", coneangle);
																	fprintf (OutPutFilePtr, "	setAttr \".ca\" %f ;\n",coneangle); 
																	break; 
											default : GetSym(); 
											} 
										 }while (sym!=RBRACE); 
 
							GetSym(); 
							break; 
 
				 default: GetSym(); 
 
	 } 
	 if(eof||sym==MEOF) 
		break; 
  } 
  // we're finished with this object 
 printf("Entering light : %s\n", ln);
  //printf ("Exiting Light\n"); 
  inLight=0; 
 
 
fflush(stdout); 
} 

// -------------------- CAMERA FUNCTION ---- 
 
void cameraobject() 
{ char cn[256];

float camrotaxis[3];
float camrotangle;

  inCamera=1;
  int times=0;
  GetSym(); 
  if(sym!=LBRACE) 
	 message(4); 
  GetSym(); 
  while(sym!=RBRACE) 
  { 
	 switch(sym) 
	 { 
		case NODE_TM:  do{ 
							  switch (sym) 
								 { 
								 case NODE_NAME:char camname[256];
													 GetSym(); 
													 if(sym==STRING) 
													 strcpy(camname, sval);strcpy(cn, camname);
													  
													 fprintf(OutPutFilePtr, "createNode transform -n \"%s\";\n", cn); 
													 //printf("Name is %s\n", camname);
 
													  CAMERALIST *ncamera;
													  CAMERALIST *currentc;
 
														  // now add this object to our list of objects... 
														ncamera=(CAMERALIST*)malloc(sizeof(CAMERALIST)); 
														ncamera->next=NULL; 
														strcpy(ncamera->name,camname); 
 
														currentc=cameraList;
														if(currentc!=NULL) 
														{ 
															while(currentc->next!=NULL) 
															currentc=currentc->next; 
															currentc->next=ncamera; 
														} 
													  else 
														 cameraList=ncamera; 
 
 
 
													 GetSym(); 
													 break; 
								 case TM_POS:float campos[3]; 
												 GetSym(); 
												 campos[0]=fval; 
												 GetSym(); 
												 campos[1]=fval; 
												 GetSym(); 
												 campos[2]=fval; 
												// GetSym(); 
												//printf ("XYZ trans Elements are : %f, %f, %f\n", campos[0], campos[2], campos[1]);
												fprintf(OutPutFilePtr, "	setAttr \".t\" -type \"double3\" %f %f %f ;\n",campos[0], campos[1], campos[2]);
												GetSym(); 
												break;

								 case TM_ROTAXIS:GetSym(); 
												 camrotaxis[0]=fval; 
												 GetSym(); 
												 camrotaxis[1]=fval; 
												 GetSym(); 
												 camrotaxis[2]=fval; 
												 GetSym(); 
												//printf ("Rot Axis Elements are : %f, %f, %f\n", camrotaxis[0], camrotaxis[1], camrotaxis[2]);
												//GetSym(); 
												break;

								 case TM_ROTANGLE:GetSym();
												  ROTXYZ NEWROTATION; 
												  camrotangle=fval; 
												  //printf ("Rot Axngle Element is : %f\n", camrotangle);

												  NEWROTATION=Quat2Deg((camrotaxis[0]),(camrotaxis[1]),(camrotaxis[2]),(camrotangle));
												  //printf(" Camera XYZ Rotation angles are : \nX=%f, Y=%f, Z=%f\n",NEWROTATION.x,NEWROTATION.y,NEWROTATION.z);
												 fprintf(OutPutFilePtr, "	setAttr \".r\" -type \"double3\" %f %f %f ;\n",NEWROTATION.x,NEWROTATION.y,NEWROTATION.z); 

												  GetSym(); 
												  break;
								 case TM_SCALE:float camscale[3]; 
												 GetSym(); 
												 camscale[0]=fval; 
												 GetSym(); 
												 camscale[1]=fval; 
												 GetSym(); 
												 camscale[2]=fval; 
												 GetSym(); 
												//printf ("XYZ scale Elements are : %f, %f, %f\n", camscale[0], camscale[1], camscale[2]);
												fprintf(OutPutFilePtr, "	setAttr \".s\" -type \"double3\" %f %f %f ;\n",camscale[0], camscale[1], camscale[2]); 
												//GetSym(); 
												break; 
								 default : GetSym(); 
								 } 
								}while (sym!=RBRACE); 
 
		  case CAMERA_SETTINGS:    CAMERALIST *currentc;
											currentc=cameraList; 
 
											if(currentc->next != NULL) 
											 currentc=currentc->next; 
											fprintf (OutPutFilePtr, "createNode camera -n \"%sShape1\" -p \"%s\";\n ",cn,cn); 
											//printf ("Camera Name from CAMERALIST = %s\n",currentc->name);
											fprintf(OutPutFilePtr, "	setAttr -k off \".v\";\n"); 
 
										 do{ 
										 switch (sym) 
											{ 
											case CAMERA_NEAR:float camnear; 
																	GetSym(); 
																	camnear=fval; 
																	//printf ("Camera Near is : %f\n", camnear);
																	fprintf (OutPutFilePtr, "	setAttr \".ncp\" %f ;\n",camnear); 
																	break; 
											case CAMERA_FAR:float camfar; 
																	GetSym(); 
																	camfar=fval; 
																	//printf ("Camera Far is : %f\n", camfar);
																	fprintf (OutPutFilePtr, "	setAttr \".fcp\" %f ;\n",camfar); 
																	break; 
											/*case CAMERA_FOV:float camFOV; 
																	GetSym(); 
																	camFOV=fval; 
																	//printf ("Camera FOV is : %f\n", camFOV);
																	//fprintf (OutPutFilePtr, "	setAttr \".ca\" %f ;\n",coneangle); 
																	break; */
											case CAMERA_TDIST:float camTDIST; 
																	GetSym(); 
																	camTDIST=fval; 
																	//printf ("Camera TDIST is : %f\n", camTDIST);
																	fprintf (OutPutFilePtr, "	setAttr \".fl\" %f ;\n",(camTDIST/4)); 
																	break; 
											default : GetSym(); 
											} 
										 }while (sym!=RBRACE); 
 
							GetSym(); 
							break; 
 
				 default: GetSym(); 
 
	 } 
	 if(eof||sym==MEOF) 
		break; 
  } 
  // we're finished with this object 
 printf("Entering camera : %s\n", cn);
  //printf ("Exiting Light\n"); 
  inCamera=0; 
  times++;
 
fflush(stdout); 
 
}

// -------------------- ANIMATION FUNCTION ----
// ANY OBJECT/CAMERA/LIGHT USING ANIMATION WILL USE THIS FUNCTION  
 
void animobj() 
{ char an[256];
//printf ("Entered Animation\n"); 
  inAnim=1;
  int counter=0;
  GetSym(); 
  if(sym!=LBRACE) 
	 message(4); 
  GetSym();
    int MaxCount=0;
  int MaxCount1=0;
 
  while(sym!=RBRACE) 
  { 
	 do{ 
		switch (sym) 
			{ 
			case NODE_NAME:char animname[256]; 
								GetSym(); 
								if(sym==STRING) 
								strcpy(animname, sval);strcpy(an, animname);
 
								//printf("ANIM Name is %s\n", animname);
 
								ANIMLIST *nanim; 
								ANIMLIST *currenta; 
 
								// now add this object to our list of objects... 
								nanim=(ANIMLIST*)malloc(sizeof(ANIMLIST)); 
								nanim->next=NULL; 
								strcpy(nanim->name,animname); 
								currenta=animList; 
								if(currenta!=NULL) 
									{ 
									while(currenta->next!=NULL) 
									currenta=currenta->next; 
									currenta->next=nanim; 
									} 
									else 
									animList=nanim; 
 
								GetSym(); 
								break; 
			case CONTROL_POS_BEZIER:	int animframe[2000];//int MaxCount1=0;
												float animpos[2000][3];
                                    printf("Found Translation\n");
 
													ANIMLIST *currenta1; 
													currenta1=animList; 
													if(currenta1->next != NULL) 
													currenta1=currenta1->next; 
 
		
												do{ 
 
												switch (sym) 
													{ 
													case CONTROL_BEZIER_POS_KEY : 
																			transkey=1; 
																			GetSym(); 
																			animframe[counter]=ival; 
																			GetSym(); 
																			animpos[counter][0]=fval; 
																			GetSym(); 
																			animpos[counter][1]=fval; 
																			GetSym(); 
																			animpos[counter][2]=fval; 
																			GetSym(); 
																			//printf ("At frame %d translate XYZ is : %f %f %f\n", (animframe[counter]/ticks), animpos[counter][0], animpos[counter][1], animpos[counter][2]);
																			//GetSym(); 
																			counter++;
																			MaxCount1++;
																			animpos[counter][0]=NULL; 
 
																			break; 
 
																		default:GetSym(); 
														} 
													}while (sym!=RBRACE); 
 
													counter=0;
													fprintf(OutPutFilePtr, "createNode animCurveTU -n \"%s_translateX\";\n",an);
													fprintf(OutPutFilePtr, "	setAttr \".tan\" 9;\n	setAttr \".wgt\" no;"); 
													fprintf(OutPutFilePtr, "	setAttr -s 2 \".ktv[0:%d]\"", (MaxCount1-1));
 													do{
													fprintf(OutPutFilePtr, " %d %f", ((animframe[counter])/ticks), (animpos[counter][0]));
													counter++; 
 													}while (animpos[counter][0]!=NULL);
													fprintf(OutPutFilePtr, ";\n\n");


													counter=0;
													fprintf(OutPutFilePtr, "createNode animCurveTU -n\"%s_translateY\";\n",an);
													fprintf(OutPutFilePtr, "	setAttr \".tan\" 9;\n	setAttr \".wgt\" no;"); 
													fprintf(OutPutFilePtr, "	setAttr -s 2 \".ktv[0:%d]\"", (MaxCount1-1));
 													do{
													fprintf(OutPutFilePtr, " %d %f", ((animframe[counter])/ticks), (animpos[counter][1]));
													counter++; 
 													}while (animpos[counter][0]!=NULL);
													fprintf(OutPutFilePtr, ";\n\n");


													counter=0;
													fprintf(OutPutFilePtr, "createNode animCurveTU-n\"%s_translateZ\";\n",an);
													fprintf(OutPutFilePtr, "	setAttr \".tan\" 9;\n	setAttr \".wgt\" no;");
													fprintf(OutPutFilePtr, "	setAttr -s 2 \".ktv[0:%d]\"", (MaxCount1-1));
													do{
													fprintf(OutPutFilePtr, " %d %f", ((animframe[counter])/ticks), (animpos[counter][2]));
													counter++;
													}while (animpos[counter][0]!=NULL);
													fprintf(OutPutFilePtr, ";\n\n");

													fprintf(OutPutFilePtr, "connectAttr \"%s_translateX.o\" \"%s.tx\";\n", an, an);
													fprintf(OutPutFilePtr, "connectAttr \"%s_translateY.o\" \"%s.ty\";\n", an, an);
													fprintf(OutPutFilePtr, "connectAttr \"%s_translateZ.o\" \"%s.tz\";\n\n", an, an);
													GetSym();

												break;

			case CONTROL_ROT_TCB :		 printf("Found Rotation\n");
												 int rotframe[2000];
												 float ROTARRAY[2000][3];
												float animrot[2000][4];
												counter=0;
												ROTXYZ NEWROTATION; 

 
													ANIMLIST *currenta3; 
													currenta3=animList; 
													if(currenta3->next != NULL) 
													currenta3=currenta3->next; 
 
													
 
												do{ 
 
												switch (sym) 
													{ 
													case CONTROL_TCB_ROT_KEY : 
																			transkey=1; 
																			GetSym(); 
																			rotframe[counter]=ival; 
																			GetSym(); 
																			animrot[counter][0]=fval; 
																			GetSym(); 
																			animrot[counter][1]=fval; 
																			GetSym(); 
																			animrot[counter][2]=fval; 
																			GetSym(); 
																			animrot[counter][3]=fval; 
																			GetSym(); 
																			//printf ("At frame %d axis of rotation is :  %f %f %f. Rotation value is : %f\n", (rotframe[counter]/ticks), animrot[counter][0], animrot[counter][1], animrot[counter][2], animrot[counter][3]);

																			NEWROTATION=Quat2Deg((animrot[counter][0]),(animrot[counter][1]),(animrot[counter][2]),(animrot[counter][3]));
																			//printf ("After Q2D : \nX = %f degrees\nY= %f degrees\nZ= %f degrees\n", NEWROTATION.x,NEWROTATION.y,NEWROTATION.z);


																			ROTARRAY[counter][0]=NEWROTATION.x;
																			ROTARRAY[counter][1]=NEWROTATION.y;
																			ROTARRAY[counter][2]=NEWROTATION.z;
																			ROTARRAY[counter+1][0]=NULL;


																			//GetSym();
																			counter++;
																			MaxCount++;
																			animrot[counter][0]=NULL; 
																			
																			break; 
 
																		default:GetSym(); 
														} 
													}while (sym!=RBRACE);


													fprintf(OutPutFilePtr, "createNode animCurveTA -n \"%s_rotateX\";\n", an);
													fprintf(OutPutFilePtr, "	setAttr \".tan\" 9;\n	setAttr \".wgt\" no;");
													fprintf(OutPutFilePtr, "	setAttr -s 2 \".ktv[0:%d]\"", (MaxCount-1));

													for (i=0;i<MaxCount;i++)
													{

                                       if (i>=1)
													ROTARRAY[i][0]=ROTARRAY[i-1][0]+ROTARRAY[i][0];

													fprintf(OutPutFilePtr, " %d %f", ((rotframe[i])/ticks), ROTARRAY[i][0]);
													//printf("X ROTATION and keyframe : %d %f\n", ((rotframe[i])/ticks), ROTARRAY[i][0]);
													}
													fprintf(OutPutFilePtr, ";\n\n");
													fprintf(OutPutFilePtr, "createNode animCurveTA -n \"%s_rotateY\";\n", an);
													fprintf(OutPutFilePtr, "	setAttr \".tan\" 9;\n	setAttr \".wgt\" no;");
													fprintf(OutPutFilePtr, "	setAttr -s 2 \".ktv[0:%d]\"", (MaxCount-1));

													for (i=0;i<MaxCount;i++)
													{

													if (i>=1)
													{
													ROTARRAY[i][1]=ROTARRAY[i-1][1]+ROTARRAY[i][1];
													float A = ROTARRAY[i][1]- ROTARRAY[i-1][1];
													float B = ROTARRAY[i][1]- ROTARRAY[i-1][1];

													  if ( ((A-B)>180) | ((B-A)>180) )
													  {
														if (A<B)
														A+=360;
														else
														B+=360;
													  }

													}

													fprintf(OutPutFilePtr, " %d %f", ((rotframe[i])/ticks), ROTARRAY[i][1]);
													//printf("Y ROTATION and keyframe : %d %f\n", ((rotframe[i])/ticks), ROTARRAY[i][1]);
													}
													fprintf(OutPutFilePtr, ";\n\n");

													fprintf(OutPutFilePtr, "createNode animCurveTA -n \"%s_rotateZ\";\n", an);
													fprintf(OutPutFilePtr, "	setAttr \".tan\" 9;\n	setAttr \".wgt\" no;");
													fprintf(OutPutFilePtr, "	setAttr -s 2 \".ktv[0:%d]\"", (MaxCount-1));

													for (i=0;i<MaxCount;i++)
													{

													if (i>=1)
													ROTARRAY[i][2]=ROTARRAY[i-1][2]+ROTARRAY[i][2];

													fprintf(OutPutFilePtr, " %d %f", ((rotframe[i])/ticks), ROTARRAY[i][2]);
													//printf("Z ROTATION and keyframe : %d %f\n", ((rotframe[i])/ticks), ROTARRAY[i][2]);
													}
													fprintf(OutPutFilePtr, ";\n\n");

													fprintf(OutPutFilePtr, "connectAttr \"%s_rotateX.o\" \"%s.rx\";\n", an, an);
													fprintf(OutPutFilePtr, "connectAttr \"%s_rotateY.o\" \"%s.ry\";\n", an, an);
													fprintf(OutPutFilePtr, "connectAttr \"%s_rotateZ.o\" \"%s.rz\";\n\n", an, an);
                                       GetSym();
												break;

			case CONTROL_SCALE_BEZIER :int scaleframe[2000];//int MaxCount=0;
												float animscale[2000][3]; 
												counter=0;
												MaxCount=0; 
                                    printf ("Found scale\n");
 
													ANIMLIST *currenta2; 
													currenta2=animList; 
													if(currenta2->next != NULL) 
													currenta2=currenta2->next; 
 
													
 
												do{ 
 
												switch (sym) 
													{ 
													case CONTROL_BEZIER_SCALE_KEY : 
																			transkey=1; 
																			GetSym(); 
																			scaleframe[counter]=ival; 
																			GetSym(); 
																			animscale[counter][0]=fval; 
																			GetSym(); 
																			animscale[counter][1]=fval; 
																			GetSym(); 
																			animscale[counter][2]=fval; 
																			GetSym(); 
																			//printf ("At frame %d scale XYZ is : %f %f %f\n", (scaleframe[counter]/ticks), animscale[counter][0], animscale[counter][1], animscale[counter][2]);
																			GetSym(); 
																			counter++;
																			MaxCount++;
																			animscale[counter][0]=NULL; 
 
																			break; 
 
																		default:GetSym(); 
														} 
													}while (sym!=RBRACE); 

													counter=0;
													fprintf(OutPutFilePtr, "createNode animCurveTU -n \"%s_scaleX\";\n", an);
													fprintf(OutPutFilePtr, "	setAttr \".tan\" 9;\n	setAttr \".wgt\" no;"); 
													fprintf(OutPutFilePtr, "	setAttr -s 2 \".ktv[0:%d]\"", (MaxCount-1));
 													do{
													fprintf(OutPutFilePtr, " %d %f", ((scaleframe[counter])/ticks), animscale[counter][0]);
													counter++; 
 													}while (animscale[counter][0]!=NULL);
													fprintf(OutPutFilePtr, ";\n\n");
													
													counter=0;
													fprintf(OutPutFilePtr, "createNode animCurveTU -n \"%s_scaleY\";\n",an);
													fprintf(OutPutFilePtr, "	setAttr \".tan\" 9;\n	setAttr \".wgt\" no;"); 
													fprintf(OutPutFilePtr, "	setAttr -s 2 \".ktv[0:%d]\"", (MaxCount-1));
 													do{
													fprintf(OutPutFilePtr, " %d %f", ((scaleframe[counter])/ticks), animscale[counter][1]);
													counter++; 
 													}while (animscale[counter][0]!=NULL);
													fprintf(OutPutFilePtr, ";\n\n");


													counter=0;
													fprintf(OutPutFilePtr, "createNode animCurveTU -n\"%s_scaleZ\";\n",an);
													fprintf(OutPutFilePtr, "	setAttr \".tan\" 9;\n	setAttr \".wgt\" no;"); 
													fprintf(OutPutFilePtr, "	setAttr -s 2 \".ktv[0:%d]\"", (MaxCount-1));
 													do{
													fprintf(OutPutFilePtr, " %d %f", ((scaleframe[counter])/ticks), animscale[counter][2]);
													counter++; 
 													}while (animscale[counter][0]!=NULL);
													fprintf(OutPutFilePtr, ";\n\n");


													fprintf(OutPutFilePtr, "connectAttr \"%s_scaleX.o\" \"%s.sx\";\n", an, an);
													fprintf(OutPutFilePtr, "connectAttr \"%s_scaleY.o\" \"%s.sy\";\n", an, an);
													fprintf(OutPutFilePtr, "connectAttr \"%s_scaleZ.o\" \"%s.sz\";\n\n", an, an);
													//GetSym();
												break; 




								 default : GetSym();
								 } 
		}while (sym!=RBRACE); 
							 
	 if(eof||sym==MEOF) 
		break; 
  } 
//printf ("Exiting Anim\n");
inLight=0; 
fflush(stdout); 
}

// -------------------- MATHS FUNCTION TO CONVERT THE MAX ----
// --------------------- ROTATIONS TO MAYA READABLE ONES -----  

ROTXYZ Quat2Deg(float origx, float origy,float origz, float origw)
{
float X,Y,Z,W;
ROTXYZ ROTATION;
X=origx;
Y=origy;
Z=origz;
W=origw;

//printf("ORIGINAL VALUES ARE : \nX = %f\nY = %f\nZ = %f\nW = %f\n\n", X,Y,Z,W);

//Convert vector and angle to a rotation matrix

float angle, dx, dy, dz;
float RtMatrix[4][4];

dx=X;
dy=Y;
dz=Z;
angle=W;

if ( (X==0)&&(Y==0)&&(Z==0)&&(W==0) )
{

//Check if any rotation on object

ROTATION.x=0;
ROTATION.y=0;
ROTATION.z=0;

return ROTATION;

}
else
  {

float length=sqrt(dx*dx+dy*dy+dz*dz);
dx/=length;
dy/=length;
dz/=length;
angle=W;

//printf("dx= %f, dy= %f,dz= %f, angle= %f\n", dx, dy, dz, angle);



	RtMatrix[0][0]=dx*dx+cos(angle)*(1-dx*dx);
	RtMatrix[1][0]=dx*dy*(1-cos(angle))-dz*sin(angle);
	RtMatrix[2][0]=dz*dx*(1-cos(angle))+dy*sin(angle);
	RtMatrix[3][0]=0;
	
	RtMatrix[0][1]=dx*dy*(1-cos(angle))+dz*sin(angle);
	RtMatrix[1][1]=dy*dy+cos(angle)*(1-dy*dy);
	RtMatrix[2][1]=dy*dz*(1-cos(angle))-dx*sin(angle);
	RtMatrix[3][1]=0;

	RtMatrix[0][2]=dz*dx*(1-cos(angle))-dy*sin(angle);
	RtMatrix[1][2]=dy*dz*(1-cos(angle))+dx*sin(angle);
	RtMatrix[2][2]=dz*dz+cos(angle)*(1-dz*dz);
	RtMatrix[3][2]=0;

	RtMatrix[0][3]=0;
	RtMatrix[1][3]=0;
	RtMatrix[2][3]=0;
	RtMatrix[3][3]=1;

// Convert Rotation matrix to Euler XYZ angles


float angle_y, angle_x, angle_z, trx, Try, trz, H, I;



    angle_y = H = -asin( RtMatrix[2][0]);        	// Calculate Y-axis angle 
    I           =  cos( angle_y );
    //angle_y    = RAD2DEG(angle_y);

    if ( fabs( I ) > 0.005 )             		// Gimball lock? 
      {
      trx      =  RtMatrix[2][2] / I;           	// No, so get X-axis angle 
      Try      = -RtMatrix[2][1]  / I;

      angle_x  = atan2( Try, trx );

      trx      =  RtMatrix[0][0] / I;           	 // Get Z-axis angle 
      Try      = -RtMatrix[1][0] / I;

      angle_z  = atan2( Try, trx );
      }
    else                                 		// Gimball lock has occurred 
      {	
      angle_x  = 0;                      		// Set X-axis angle to zero 

      trx      = RtMatrix[1][1];                 	// And calculate Z-axis angle 
      Try      = RtMatrix[0][1];

      angle_z  = atan2( Try, trx );
      }

//printf ("\nAngle AX = %f radians\n", angle_x);

angle_x=RAD2DEG(angle_x);
angle_y=RAD2DEG(angle_y);
angle_z=RAD2DEG(angle_z);

//printf ("\nAngle X = %f degress\n", angle_x );
//printf ("Angle Y = %f degress\n", angle_y );
//printf ("Angle Z = %f degress\n", angle_z );

ROTATION.x=(angle_x*-1);
ROTATION.y=angle_y;
ROTATION.z=(angle_z*-1);

return ROTATION;
  } 


}

// -------------------- SCENE INFORMATION FUNCTION ----
// ------ e.g. FRAMES PER SECOND, START AND END FRAMES, etc. ----   
 
void sceneinfo() 
{ 
//printf ("Entered scene info\n");
  GetSym(); 
  if(sym!=LBRACE) 
	 message(4); 
  GetSym(); 
  while(sym!=RBRACE) 
  { 
	 switch(sym) 
	 { 
		case SCENE_TICKSPERFRAME :	GetSym(); 
											ticks=ival; 
											//printf("Ticks speed is %d\n", ticks);
											GetSym(); 
											break; 
		case SCENE_FIRSTFRAME :    GetSym(); 
											SF=ival; 
											//printf("SF is %d\n", SF);
											GetSym(); 
											break; 
		case SCENE_LASTFRAME :     GetSym(); 
											EF=ival; 
											//printf("EF is %d\n", EF);
											GetSym(); 
											break; 
		case SCENE_FRAMESPEED :    GetSym(); 
											FS=ival; 
											//printf("FS is %d\n", FS);
											GetSym(); 
											break; 
 
		default:GetSym(); 
	 } 
  } 
} 

// -------------------- FUNCTION TO PRINT RELEVANT STUFF ----
// ----------------------- AT THE TOP OF THE .MA FILE ------- 
 
void top() 
{ 
  const char * const weekday[]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"}; 
  const char * const month[]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"}; 
//  int this_hour; 
 
  time_t now; 
  struct tm *date; 
 
  time(&now); 
  date=localtime(&now); 
 
  fprintf (OutPutFilePtr, "//Maya ASCII 3.0 scene\n"); 
  fprintf (OutPutFilePtr, "//Name: %s\n", outputfile); 
 
  fprintf (OutPutFilePtr, "//Last modified:%s, ", weekday[date->tm_wday]); 
  if(date->tm_mday<10) 
	 fprintf (OutPutFilePtr, "%s 0%d, ", month[date->tm_mon],date->tm_mday); 
  else 
	 fprintf (OutPutFilePtr, "%s %2d, ", month[date->tm_mon],date->tm_mday); 
  fprintf (OutPutFilePtr, "%4d ", date->tm_year+1900); 
  if((date->tm_hour%12<10)&&(date->tm_hour%12>0)) 
	 fprintf (OutPutFilePtr, "0%d:",date->tm_hour%12); 
  else 
	 fprintf (OutPutFilePtr, "%2d:",date->tm_hour%12==0?12:date->tm_hour%12); 
  if(date->tm_min<10) 
	 fprintf (OutPutFilePtr, "0%d:", date->tm_min); 
  else 
	 fprintf (OutPutFilePtr, "%2d:", date->tm_min); 
  if(date->tm_sec<10) 
	 fprintf (OutPutFilePtr, "0%d ", date->tm_sec); 
  else 
	 fprintf (OutPutFilePtr, "%2d ", date->tm_sec); 
  fprintf (OutPutFilePtr, "%s\n",date->tm_hour<12?"AM":"PM"); 
 
  fprintf (OutPutFilePtr, "requires maya \"3.0\";\n"); 
  fprintf (OutPutFilePtr, "currentUnit -l millimeter -a degree -t pal;\n\n"); 
} 
 
// -------------------- FUNCTION TO PRINT RELEVANT STUFF ----
// --------------------- AT THE BOTTOM OF THE .MA FILE ------- 
 
void bottom() 
{
 
fprintf(OutPutFilePtr, "createNode script -n \"animationScriptNode\";\n	setAttr \".a\" -type \"string\" (\n"); 
fprintf(OutPutFilePtr, "		\"playbackOptions -min %d -max %d -animationStartTime %d -animationEndTime %d ;\");\n", SF, EF, SF, EF);  
 
 
  OLIST *current; 
  char bottomblurb[]="\n\n createNode lightLinker -n \"lightLinker1\";\n createNode displayLayerManager -n \"layerManager\";\n createNode displayLayer -n \"defaultLayer\";\n createNode renderLayerManager -n \"renderLayerManager\";\n createNode renderLayer -n \"defaultRenderLayer\";\n createNode renderLayer -s -n \"globalRender\";\n createNode partition -n \"mtorPartition\";\n connectAttr \":defaultLightSet.msg\" \"lightLinker1.lnk[0].llnk\";\n connectAttr \":initialShadingGroup.msg\" \"lightLinker1.lnk[0].olnk\";\n connectAttr \":defaultLightSet.msg\" \"lightLinker1.lnk[1].llnk\";\n connectAttr \":initialParticleSE.msg\" \"lightLinker1.lnk[1].olnk\";\n connectAttr \"layerManager.dli[0]\" \"defaultLayer.id\";\n connectAttr \"renderLayerManager.rlmi[0]\" \"defaultRenderLayer.rlid\";\n connectAttr \":defaultRenderGlobals.msg\" \"mtorPartition.msg\";\n connectAttr \"lightLinker1.msg\" \":lightList1.ln\" -na;\n"; 
 
  fprintf (OutPutFilePtr, "%s", bottomblurb); 
  fflush(OutPutFilePtr); 
 
//  printf("done that"); 
  fflush(stdout); 
  current=objList; 
  while(current!=NULL) 
  { 
	 fprintf (OutPutFilePtr, "  connectAttr \"%sShape1.iog\" \":initialShadingGroup.dsm\" -na;\n",current->name); 
	 current=current->next; 
  } 
 
  LIGHTLIST *currentl; 
	 currentl=lightList; 
  while(currentl!=NULL) 
  { 
	 fprintf (OutPutFilePtr, "  connectAttr \"%sShape1.ltd\" \":lightList1.l\" -na;\n",currentl->name); 
	 fprintf (OutPutFilePtr, "  connectAttr \"%sShape1.iog\" \":defaultLightSet.dsm\" -na;\n",currentl->name); 
	 currentl=currentl->next;
  } 
 

  fprintf (OutPutFilePtr, "// End of %s ",outputfile); 
}

// -------------------- FUNCTION TO CREATE THE INITIAL ---- 
// ------------------------- STATE OF THE OBJECT ----------  
 
void WriteObjName() 
{ 
  OLIST *nobj; 
  OLIST *current; 
  char objblurb1[]="createNode transform -n"; 
  char objblurb3[]="createNode mesh -n \""; 
  char objblurb4[]="Shape1\" -p \""; 
  char objblurb5[]="\";"; 
 
//  char objblurb2[]="setAttr -k off \".v\";\n setAttr \".vir\" yes;\n setAttr \".vif\" yes;\n setAttr \".uvst[0].uvsn\" -type \"string\" \"map1\";\n "; 
 
  strcpy (name, sval);
 
 
  fprintf (OutPutFilePtr, " %s \"%s\";\n %s%s%s%s%s\n", objblurb1, name, objblurb3, name, objblurb4, name, objblurb5);  // place in file 
 
  // now add this object to our list of objects... 
  nobj=(OLIST*)malloc(sizeof(OLIST)); 
  nobj->next=NULL; 
  strcpy(nobj->name,name); 
 
  current=objList; 
  if(current!=NULL) 
  { 
	 while(current->next!=NULL) 
		current=current->next; 
	 current->next=nobj; 
  } 
  else 
	 objList=nobj; 
 
 
//  printf ("Object Name = %s\n", name);

  fflush(stdout); 
 
}

// -------------------- ERROR MESSAGE FUNCTION ----  
 
 void message(int msg) 
{ 
  switch(msg) 
  { 
	 case  0: fprintf(stdout,"Unknown Message!\n"); 
				 break; 
	 case  1: fprintf(stderr,"Usage: Max2Maya <Input File> <Output File> optional : -h for help\n");
					  exit(EXIT_FAILURE);
	 case  2: fprintf(stderr,"Unexpected EOF"); 
					  exit(EXIT_FAILURE); 
	 case  4: fprintf(stderr,"Some Fatal Error just occurred"); 
					  exit(EXIT_FAILURE); 
	 case  5: fprintf(stderr,"Invalid Input File"); 
					  exit(EXIT_FAILURE); 
	  case  6: fprintf(stderr,"Error: Unable to open/create Output File\n"); 
				  exit(EXIT_FAILURE); 
	  case  7: fprintf(stderr,"Error: Unable to open Input File\n"); 
				  exit(EXIT_FAILURE); 
	 default: fprintf(stdout,"Unknown Message!\n"); 
				 break; 
  } 
}

// -------------------- HELP FUNCTION ---- 

void help()
{
printf ("\nMax2Maya %s is a program to convert between 3DStudio Max and Maya written by :\nDavid Walker NCCA 2000\n\n", version);
printf ("Usage : Max2Maya <Input File> <Output File> optional : -u -g -a -l -c -s -h\n");
printf ("-u ..... Universal, i.e. take all supported info across platforms\n-g ..... Take geometry info\n-a ..... Take animation info\n-l ..... Take lights and light info");
printf ("\n-c ..... Take cameras and camera info\n-s ..... Take scene info\n-h ..... This help file\n");
exit(EXIT_FAILURE); 
}

// ----------- PARSING FUNCTION INITIALISATION ---- 
 
void initialise() 
//  initialise the special characters for recognition by the parser.
{ 
  special['{']=LBRACE; 
  special['}']=RBRACE; 
  special['*']=STAR; 
  special[':']=COLON; 
  special['"']=SPEECH; 
  special['.']=FULL; 
} 


 
void GetCh() 
// GetCh   - get character  - Eike Anderson, 2000
//
// retrieves the next character from the source file
// and stores it in the "ch" global variable.
//
// if the end of the source file is reached, the eof
// flag is set to true and '' is returned as the next
// character
{ 
  if(feof(InputFilePtr)) 
  { 
  printf("%s Completed\n", filename);
  printf("%s Generated\n", outputfile);
  fflush(stdout); 
	 eof=1; 
	 ch=''; 
	 return; 
  } 
  if(cc==ll) 
  { 
	 if(feof(InputFilePtr)) 
	 { 
		if(inObj) 
		  message(2); 
		else 
		{ 
  printf("eof at new line detected\n"); 
  fflush(stdout); 
		  eof=1; 
		  ch=''; 
		} 
		return; 
	 } 
	 cc=0; 
	 fgets(line,LINELEN-1,InputFilePtr); 
	 ll=strlen(line); 
  } 
//  printf("before..."); 
//  fflush(stdout); 
  ch=line[cc++]; 
//  printf("after\n"); 
//  fflush(stdout); 
  if(ch==CR||ch==LF||isspace(ch)) 
	 ch=' '; 
} 
 
void GetSym()
// GetSym   - get symbol  - Eike Anderson, 2000
//
// retrieves the next symbol from the source file and
// stores it in the "sym" global variable. If the symbol
// is a data value, the value is stored in the corresponding
// global data variable.
// if the symbol is a token/keyword from the known_tokens
// enumerated datatype the corresponding symbol value from
// the symbol enumerated datatype is selected for storage in
// the "sym" global variable
// special characters are parsed correctly by this function
{ 
  int index; 
/*  if(eof||ch=='') 
  { 
	 sym=MEOF; 
	 return; 
  }*/ 
  while(ch==' ')      //  get rid of leading spaces 
	 GetCh(); 
  if(isdigit(ch)||(ch=='-')) 
  { 
	 int    neg=1; 
	 char   fstop=0; 
	 double num=0,newval=0; 
	 int    nv,div=1; 
	 if(ch=='-') 
	 { 
		neg=-1; 
		GetCh(); 
	 } 
	 do 
	 { 
		if(ch=='.') 
			fstop=1; 
		else 
		{ 
			nv=ch-'0'; 
			newval=(double)nv; 
			if(fstop) 
			{ 
			  div*=10; 
			  num+=(double)(newval/div); 
			} 
			else 
			  num=num*10+newval; 
		} 
		GetCh(); 
	 } 
	 while(isdigit(ch)||(ch=='.')); 
	 if(fstop) 
	 { 
		sym=REAL; 
		fval=(double)num*neg; 
//printf("%f\n",fval); 
fflush(stdout); 
	 } 
	 else 
	 { 
		sym=INTEGER; 
		ival=(long int)num*neg; 
//printf("%d\n",ival); 
//fflush(stdout); 
	 } 
  } 
  else 
  { 
	 switch(ch) 
	 { 
		case '': sym=MEOF; 
//printf("eof\n"); 
fflush(stdout); 
					 break; 
		case '"': sym=STRING; 
					 index=0; 
					 GetCh(); 
					 if(ch=='"') 
					 { 
						strcpy(sval," "); 
						GetCh(); 
						break; 
					 } 
					 do 
					 { 
						 if(ch=='"') 
							sval[index]='\0'; 
						 else 
							sval[index]=ch; 
						 index++; 
						 GetCh(); 
					 }while(ch!='"'); 
					 if(ch=='"') 
						sval[index]='\0'; 
					 GetCh(); 
					 break; 
		case '*': index=0; 
					 sym=NIL;        // unknown token by default 
					 GetCh(); 
					 if(isalnum(ch)) 
					 { 
						do 
						{ 
						  if(index<128) 
							 id[index++]=ch; 
						  GetCh(); 
						}while(isalnum(ch)||(ch=='_')); 
						id[index]='\0'; 
//printf("%s\n",id); 
fflush(stdout); 
						for(index=0;index<KNOWN_TOKENS;index++) 
						{ 
						  if(strcmp(id,word[index])==0) 
							 sym=symbol[index]; 
						} 
					 } 
					 break; 
		default:  sym=special[ch]; 
//printf("%c\n",ch); 
fflush(stdout); 
					 GetCh(); 
					 break; 
	 } 
  } 
} 
